@@ -13,7 +13,7 @@ impl IntoIterator for Tiles {
     type IntoIter = std::iter::Map<BitIter<u64>, fn(u8) -> Coord>;
 
     fn into_iter(self) -> Self::IntoIter {
-        BitIter::new(self.0).map(|i| Coord::from_sparse_i(i as u8))
+        BitIter::new(self.0).map(|i| Coord(i as u8))
     }
 }
 
@@ -40,11 +40,11 @@ impl Tiles {
     }
 
     pub fn coord(coord: Coord) -> Tiles {
-        Tiles(1 << coord.sparse_i())
+        Tiles(1 << coord.0)
     }
 
     pub fn has(self, coord: Coord) -> bool {
-        (self.0 >> coord.sparse_i()) & 1 != 0
+        (self.0 >> coord.0) & 1 != 0
     }
 
     pub fn is_empty(self) -> bool {
@@ -56,73 +56,76 @@ impl Tiles {
     }
 
     pub fn get_nth(self, index: u32) -> Coord {
-        Coord::from_sparse_i(get_nth_set_bit(self.0, index))
+        Coord(get_nth_set_bit(self.0, index))
     }
 
     #[must_use]
     pub fn set(self, coord: Coord) -> Self {
-        Tiles(self.0 | (1 << coord.sparse_i()))
+        Tiles(self.0 | (1 << coord.0))
     }
 
     #[must_use]
     pub fn clear(self, coord: Coord) -> Self {
-        Tiles(self.0 & !(1 << coord.sparse_i()))
+        Tiles(self.0 & !(1 << coord.0))
     }
 
     pub fn not(self, size: u8) -> Self {
         Tiles(!self.0) & Tiles::full(size)
     }
 
-    pub fn left(self, size: u8) -> Self {
-        Tiles(self.0 >> 1) & Tiles::full(size)
+    fn left(self) -> Self {
+        Tiles((self.0 >> 1) & 0x7f7f7f7f7f7f7f7f)
     }
 
-    pub fn right(self, size: u8) -> Self {
-        Tiles(self.0 << 1) & Tiles::full(size)
+    fn right(self) -> Self {
+        Tiles((self.0 << 1) & 0xfefefefefefefefe)
     }
 
-    pub fn down(self, size: u8) -> Self {
-        Tiles(self.0 >> 8) & Tiles::full(size)
+    fn down(self) -> Self {
+        Tiles((self.0 >> 8) & 0x00ffffffffffffff)
     }
 
-    pub fn up(self, size: u8) -> Self {
-        Tiles(self.0 << 8) & Tiles::full(size)
+    fn up(self) -> Self {
+        Tiles((self.0 << 8) & 0xffffffffffffff00)
     }
 
     pub fn copy_targets(self, size: u8) -> Self {
         // counterclockwise starting from left
-        self.left(size)
-            | self.left(size).down(size)
-            | self.down(size)
-            | self.right(size).down(size)
-            | self.right(size)
-            | self.right(size).up(size)
-            | self.up(size)
-            | self.left(size).up(size)
+        let result = self.left()
+            | self.left().down()
+            | self.down()
+            | self.right().down()
+            | self.right()
+            | self.right().up()
+            | self.up()
+            | self.left().up();
+        result & Tiles::full(size)
     }
 
     pub fn jump_targets(self, size: u8) -> Self {
         // counterclockwise starting from left.left
-        self.left(size).left(size)
-            | self.left(size).left(size).down(size)
-            | self.left(size).left(size).down(size).down(size)
-            | self.left(size).down(size).down(size)
-            | self.down(size).down(size)
-            | self.right(size).down(size).down(size)
-            | self.right(size).right(size).down(size).down(size)
-            | self.right(size).right(size).down(size)
-            | self.right(size).right(size)
-            | self.right(size).right(size).up(size)
-            | self.right(size).right(size).up(size).up(size)
-            | self.right(size).up(size).up(size)
-            | self.up(size).up(size)
-            | self.left(size).up(size).up(size)
-            | self.left(size).left(size).up(size).up(size)
-            | self.left(size).left(size).up(size)
+        let result = self.left().left()
+            | self.left().left().down()
+            | self.left().left().down().down()
+            | self.left().down().down()
+            | self.down().down()
+            | self.right().down().down()
+            | self.right().right().down().down()
+            | self.right().right().down()
+            | self.right().right()
+            | self.right().right().up()
+            | self.right().right().up().up()
+            | self.right().up().up()
+            | self.up().up()
+            | self.left().up().up()
+            | self.left().left().up().up()
+            | self.left().left().up();
+
+        result & Tiles::full(size)
     }
 
     pub fn map(self, size: u8, sym: D4Symmetry) -> Tiles {
-        assert!(size < AtaxxBoard::MAX_SIZE);
+        assert!(size <= AtaxxBoard::MAX_SIZE);
 
         let mut result = Tiles::empty();
         for c in self {
@@ -172,3 +175,33 @@ impl std::ops::BitAndAssign for Tiles {
         self.0 &= rhs.0
     }
 }
+
+#[cfg(test)]
+mod tests {
+    use crate::games::ataxx::Coord;
+
+    use super::Tiles;
+
+    #[test]
+    fn copy_jump() {
+        let cases = [
+            (8, Coord::from_xy(7, 7), 0x40c0000000000000, 0x2020e00000000000),
+            (8, Coord::from_xy(0, 7), 0x0203000000000000, 0x0404070000000000),
+            (8, Coord::from_xy(7, 0), 0x000000000000c040, 0x0000000000e02020),
+            (8, Coord::from_xy(0, 0), 0x0000000000000302, 0x0000000000070404),
+        ];
+
+        for (size, coord, copy, jump) in cases {
+            println!("Size {}, coord {:?}", size, coord);
+
+            let actual_copy = Tiles::coord(coord).copy_targets(size);
+            println!("{}", actual_copy);
+
+            let actual_jump = Tiles::coord(coord).jump_targets(size);
+            println!("{}", actual_jump);
+
+            assert_eq!(actual_copy, Tiles(copy), "Wrong copy targets");
+            assert_eq!(actual_jump, Tiles(jump), "Wrong jump targets");
+        }
+    }
+}
