@@ -1,23 +1,22 @@
+use std::cmp::{max, Ordering};
+
+use chess::{Piece, ALL_PIECES};
+
 use crate::ai::minimax::Heuristic;
-use crate::games::chess::ChessBoard;
 use crate::ai::solver::SolverHeuristic;
 use crate::board::Board;
-use chess::{ALL_PIECES, Piece};
+use crate::games::chess::ChessBoard;
 
 #[derive(Debug)]
-struct ChessPieceValueHeuristic;
+pub struct ChessPieceValueHeuristic;
 
 impl Heuristic<ChessBoard> for ChessPieceValueHeuristic {
     type V = i32;
 
-    fn bound(&self) -> i32 {
-        i32::MAX
-    }
-
     fn value(&self, board: &ChessBoard, length: u32) -> Self::V {
         if board.is_done() {
-            return SolverHeuristic.value(board, length);
-        };
+            return SolverHeuristic.value(board, length).to_i32();
+        }
 
         let mut total = 0;
 
@@ -31,13 +30,19 @@ impl Heuristic<ChessBoard> for ChessPieceValueHeuristic {
             };
 
             for square in *board.inner().pieces(piece) {
-                let matches = board.inner().color_on(square).unwrap() == board.inner().side_to_move();
-                let sign = if matches { 1 } else { -1 };
-
-                total += value * sign;
+                // SAFETY: unwrap is safe because `square` contains a piece.
+                if board.inner().color_on(square).unwrap() == board.inner().side_to_move() {
+                    total += value;
+                } else {
+                    total -= value;
+                }
             }
         }
 
         total
     }
-}
\ No newline at end of file
+
+    fn merge(old: Self::V, new: Self::V) -> (Self::V, Ordering) {
+        (max(old, new), new.cmp(&old))
+    }
+}
