@@ -1,4 +1,7 @@
+use std::ops::ControlFlow;
+
 use cast_trait::Cast;
+use internal_iterator::{InternalIterator, IntoInternalIterator};
 
 use crate::board::{Outcome, Player};
 
@@ -33,16 +36,28 @@ pub trait Flip {
 
 impl OutcomeWDL {
     /// Convert this to a WDL with a one at the correct place and zero otherwise.
-    pub fn to_wdl<V: num::One + num::Zero>(self) -> WDL<V> {
+    pub fn to_wdl<V: num_traits::One + num_traits::Zero>(self) -> WDL<V> {
         match self {
-            OutcomeWDL::Win => WDL { win: V::one(), draw: V::zero(), loss: V::zero() },
-            OutcomeWDL::Draw => WDL { win: V::zero(), draw: V::one(), loss: V::zero() },
-            OutcomeWDL::Loss => WDL { win: V::zero(), draw: V::zero(), loss: V::one() },
+            OutcomeWDL::Win => WDL {
+                win: V::one(),
+                draw: V::zero(),
+                loss: V::zero(),
+            },
+            OutcomeWDL::Draw => WDL {
+                win: V::zero(),
+                draw: V::one(),
+                loss: V::zero(),
+            },
+            OutcomeWDL::Loss => WDL {
+                win: V::zero(),
+                draw: V::zero(),
+                loss: V::one(),
+            },
         }
     }
 
     /// Convert a win to `1`, draw to `0` and loss to `-1`.
-    pub fn sign<V: num::Zero + num::One + std::ops::Neg<Output=V>>(self) -> V {
+    pub fn sign<V: num_traits::Zero + num_traits::One + std::ops::Neg<Output = V>>(self) -> V {
         match self {
             OutcomeWDL::Win => V::one(),
             OutcomeWDL::Draw => V::zero(),
@@ -51,7 +66,7 @@ impl OutcomeWDL {
     }
 
     /// Convert a win to `inf`, draw to `0` and loss to `-inf`.
-    pub fn inf_sign<V: num::Float>(self) -> V {
+    pub fn inf_sign<V: num_traits::Float>(self) -> V {
         match self {
             OutcomeWDL::Win => V::infinity(),
             OutcomeWDL::Draw => V::zero(),
@@ -59,28 +74,47 @@ impl OutcomeWDL {
         }
     }
 
-    /// Combine outcomes together in minimax-style.. `None` means unknown.
-    pub fn best<I: IntoIterator<Item=Option<OutcomeWDL>>>(children: I) -> Option<OutcomeWDL> {
+    /// The reverse of [Outcome::pov].
+    pub fn un_pov(self, pov: Player) -> Outcome {
+        match self {
+            OutcomeWDL::Win => Outcome::WonBy(pov),
+            OutcomeWDL::Draw => Outcome::Draw,
+            OutcomeWDL::Loss => Outcome::WonBy(pov.other()),
+        }
+    }
+
+    /// Pick the best possible outcome, assuming `Win > Draw > Loss`.
+    /// Make sure to flip the child values as appropriate, this function assumes everything is form the parent POV.
+    pub fn best<I: IntoInternalIterator<Item = OutcomeWDL>>(children: I) -> OutcomeWDL {
+        Self::best_maybe(children.into_internal_iter().map(Some)).unwrap()
+    }
+
+    /// Pick the best possible outcome, assuming `Some(Win) > None > Some(Draw) > Some(Loss)`.
+    /// Make sure to flip the child values as appropriate, this function assumes everything is form the parent POV.
+    pub fn best_maybe<I: IntoInternalIterator<Item = Option<OutcomeWDL>>>(children: I) -> Option<OutcomeWDL> {
         let mut any_unknown = false;
         let mut all_known_are_loss = true;
 
-        for outcome in children {
-            match outcome {
+        let control = children.into_internal_iter().try_for_each(|child| {
+            match child {
                 None => {
                     any_unknown = true;
                 }
                 Some(OutcomeWDL::Win) => {
-                    //early exit, we've found a win
-                    return Some(OutcomeWDL::Win);
+                    return ControlFlow::Break(());
                 }
                 Some(OutcomeWDL::Draw) => {
                     all_known_are_loss = false;
                 }
                 Some(OutcomeWDL::Loss) => {}
             }
-        }
 
-        if any_unknown {
+            ControlFlow::Continue(())
+        });
+
+        if let ControlFlow::Break(()) = control {
+            Some(OutcomeWDL::Win)
+        } else if any_unknown {
             None
         } else if all_known_are_loss {
             Some(OutcomeWDL::Loss)
@@ -90,14 +124,38 @@ impl OutcomeWDL {
     }
 }
 
-impl<V: num::Float> WDL<V> {
+impl<V: num_traits::Float> WDL<V> {
     pub fn nan() -> WDL<V> {
-        WDL { win: V::nan(), draw: V::nan(), loss: V::nan() }
+        WDL {
+            win: V::nan(),
+            draw: V::nan(),
+            loss: V::nan(),
+        }
+    }
+}
+
+impl<V> WDL<V> {
+    pub fn new(win: V, draw: V, loss: V) -> Self {
+        WDL { win, draw, loss }
+    }
+
+    pub fn to_slice(self) -> [V; 3] {
+        [self.win, self.draw, self.loss]
+    }
+}
+
+impl<V: num_traits::One + num_traits::Zero + PartialEq> WDL<V> {
+    pub fn try_to_outcome_wdl(self) -> Option<OutcomeWDL> {
+        let outcomes = [OutcomeWDL::Win, OutcomeWDL::Draw, OutcomeWDL::Loss];
+        outcomes.iter().copied().find(|&o| o.to_wdl() == self)
     }
 }
 
 impl<V: Copy> WDL<V> {
-    pub fn cast<W>(self) -> WDL<W> where V: Cast<W> {
+    pub fn cast<W>(self) -> WDL<W>
+    where
+        V: Cast<W>,
+    {
         WDL {
             win: self.win.cast(),
             draw: self.draw.cast(),
@@ -106,13 +164,13 @@ impl<V: Copy> WDL<V> {
     }
 }
 
-impl<V: Copy + std::ops::Sub<V, Output=V>> WDL<V> {
+impl<V: Copy + std::ops::Sub<V, Output = V>> WDL<V> {
     pub fn value(self) -> V {
         self.win - self.loss
     }
 }
 
-impl<V: Copy + std::ops::Add<V, Output=V>> WDL<V> {
+impl<V: Copy + std::ops::Add<V, Output = V>> WDL<V> {
     pub fn sum(self) -> V {
         self.win + self.draw + self.loss
     }
@@ -135,11 +193,13 @@ impl POV for Outcome {
     type Output = OutcomeWDL;
     fn pov(self, pov: Player) -> OutcomeWDL {
         match self {
-            Outcome::WonBy(player) => if player == pov {
-                OutcomeWDL::Win
-            } else {
-                OutcomeWDL::Loss
-            },
+            Outcome::WonBy(player) => {
+                if player == pov {
+                    OutcomeWDL::Win
+                } else {
+                    OutcomeWDL::Loss
+                }
+            }
             Outcome::Draw => OutcomeWDL::Draw,
         }
     }
@@ -157,11 +217,15 @@ impl Flip for OutcomeWDL {
 
 impl<V: Copy> Flip for WDL<V> {
     fn flip(self) -> Self {
-        WDL { win: self.loss, draw: self.draw, loss: self.win }
+        WDL {
+            win: self.loss,
+            draw: self.draw,
+            loss: self.win,
+        }
     }
 }
 
-impl<V: Copy + std::ops::Add<V, Output=V>> std::ops::Add<WDL<V>> for WDL<V> {
+impl<V: Copy + std::ops::Add<V, Output = V>> std::ops::Add<WDL<V>> for WDL<V> {
     type Output = WDL<V>;
 
     fn add(self, rhs: WDL<V>) -> Self::Output {
@@ -173,7 +237,7 @@ impl<V: Copy + std::ops::Add<V, Output=V>> std::ops::Add<WDL<V>> for WDL<V> {
     }
 }
 
-impl<V: Copy + std::ops::Sub<V, Output=V>> std::ops::Sub<WDL<V>> for WDL<V> {
+impl<V: Copy + std::ops::Sub<V, Output = V>> std::ops::Sub<WDL<V>> for WDL<V> {
     type Output = WDL<V>;
 
     fn sub(self, rhs: WDL<V>) -> Self::Output {
@@ -185,16 +249,38 @@ impl<V: Copy + std::ops::Sub<V, Output=V>> std::ops::Sub<WDL<V>> for WDL<V> {
     }
 }
 
-impl<V: Copy + std::ops::Add<V, Output=V>> std::ops::AddAssign<WDL<V>> for WDL<V> {
+impl<V: Copy + std::ops::Add<V, Output = V>> std::ops::AddAssign<WDL<V>> for WDL<V> {
     fn add_assign(&mut self, rhs: WDL<V>) {
         *self = *self + rhs;
     }
 }
 
-impl<V: Copy + std::ops::Div<V, Output=V>> std::ops::Div<V> for WDL<V> {
+impl<V: Copy + std::ops::Mul<V, Output = V>> std::ops::Mul<V> for WDL<V> {
+    type Output = WDL<V>;
+
+    fn mul(self, rhs: V) -> Self::Output {
+        WDL {
+            win: self.win * rhs,
+            draw: self.draw * rhs,
+            loss: self.loss * rhs,
+        }
+    }
+}
+
+impl<V: Copy + std::ops::Div<V, Output = V>> std::ops::Div<V> for WDL<V> {
     type Output = WDL<V>;
 
     fn div(self, rhs: V) -> Self::Output {
-        WDL { win: self.win / rhs, draw: self.draw / rhs, loss: self.loss / rhs }
+        WDL {
+            win: self.win / rhs,
+            draw: self.draw / rhs,
+            loss: self.loss / rhs,
+        }
+    }
+}
+
+impl<V: Default + Copy + std::ops::Add<Output = V>> std::iter::Sum for WDL<V> {
+    fn sum<I: Iterator<Item = Self>>(iter: I) -> Self {
+        iter.fold(Self::default(), |a, v| a + v)
     }
 }
